/*

 Cheap DSP dsPICradio board LCD routines
 This is polling driver with 8-bit interface

 Juha Niinikoski OH2NLT 26.08.2005

 Modified for Digital RX hardware 12.09.2006
 MPLAB C30 version, OH2GWE, 2006.10.17

 IRQ mask added to LCD I/O port operations. IRQ tone output needs this protection.
 S-meter roll over corrected, interface changed to int 02.12.2006
 LCD HEX output added 09.07.2007
 PA100 temperature meter degrees sign added to the soft fonts #4 16.11.2008
 
 Removed the include statement for the file lcd-trx2.h. As there are only 5 definitions,
 and these definitions are never used anywhere else, it made more sense to include them
 in this file. - 5B4AIY 12-AUG-11
 
 Modified lcd_putchhex() to use conditional, which is more concise - 5B4AIY 28/AUG/11
 Modified lcd_putst() to use a more concise form - 5B4AIY 28/AUG/11
*/

#define LINE1	0x00		// Line 1 start address
#define LINE2	0x40		// Line 2 start address

#define CURSET	0x80		// Set memory cursor
#define	CLEAR	0x01		// Clear display
#define ADRSET	0x40		// Set CGRAM address

// External data
extern unsigned char lcdpbuff[20];

// External functions
extern void us_delay(unsigned int);
extern void ms_delay(unsigned int);

// Includes
#include "juma-pa100.h"
#include <stdio.h>

void clk_lcd(unsigned char ldata)			// Set data & clock both LCD controllers
	{
	__asm__ volatile ("disi #9");			// Protect IRQ tone output to RD1 I/O
	LCD_DATA = (LCD_DATA & ~LCD_MASK) | ((ldata << LCD_SHIFT) & LCD_MASK);	// Set LCD data lines
	us_delay(1);							// Extra delay for filter board latch wires
	LCD_E = 1;
	us_delay(1);
	LCD_E = 0;
	us_delay(1); 							// Some delay
	}

void wait_lcd_rdy(void)						// Wait until LCD module is ready
	{
	unsigned int BF_AC;						// BF flag & address counter

	BF_AC = (0x80 << LCD_SHIFT);			// Set busy for first round
	__asm__ volatile ("disi #4");			// Protect IRQ tone output to RD1 I/O
	LCD_TRIS = LCD_TRIS | LCD_MASK;			// Make data port input, LCD bus direction bits = 1
	LCD_RW = 1;								// Read
	LCD_RS = 0;								// Command mode

	while(BF_AC & (0x80 << LCD_SHIFT))
		{
		us_delay(1);						// 18.01.2004 for slow display
		LCD_E = 1;
		us_delay(2);
		BF_AC = LCD_BUS & LCD_MASK;			// Read from port pins
		LCD_E = 0;
		}

	LCD_RW = 0;
	LCD_RS = 1;
	__asm__ volatile ("disi #4");			// Protect IRQ tone output to RD1 I/O
	LCD_TRIS = LCD_TRIS & (~LCD_MASK);		// Return to write mode
	}

void set_cur_lcd(unsigned char cur)			// Set cursor position at selected display
	{
	wait_lcd_rdy( );
	LCD_RS = 0;								// Switch command mode		
	clk_lcd(cur | CURSET);					// Set cursor command + cursor position
	LCD_RS = 1;								// Back to data mode
	}

void clear_lcd(void)						// Clear selected display
	{
	wait_lcd_rdy( );
	LCD_RS = 0;								// Switch command mode		
	clk_lcd(CLEAR);							// Clear display. Display going to be busy 1,6 mS
	LCD_RS = 1;								// Back to data mode
	}

void initlcd(void)		// Initialise selected LCD controller
	{
	LCD_RS = 0;			// Set command mode
	LCD_RW = 0;
	ms_delay(15);		// Start delay
	clk_lcd(0x38);		// Write initialisation data with delays
	ms_delay(5);
	clk_lcd(0x38);
	ms_delay(1);
	clk_lcd(0x38);		// 8-bit, 2 rows, 5x7 matrix, display on
	ms_delay(1);		// Cursor off, no blink
	clk_lcd(0x0C);
	ms_delay(1);
	clk_lcd(0x01);
	ms_delay(2);
	clk_lcd(0x06);		// Increment, no display shift
	ms_delay(1);
	LCD_RS = 1;			// Set data mode
	}

void lcdoutch(unsigned char lcd_char)	// Output character to selected display
	{
	wait_lcd_rdy();
	clk_lcd(lcd_char);	
	}

/* HD44780 character generator routines */
/* Modified for SOLOMON LM1125SYLU1 display */

const char Blockslr[6] = {0x20, 0x00, 0x00, 0x01, 0x01, 0x02};	// Blocks for L to R bar graph
const char Blocksrl[6] = {0x20, 0x03, 0x03, 0x04, 0x04, 0x02};	// R to L

void set_cgram(unsigned char adr, unsigned char dta)	// Set character generator RAM address & data
	{
	wait_lcd_rdy( );
	LCD_RS = 0;				// Switch command mode
	adr = adr & 0x3F;
	clk_lcd(adr | ADRSET);	// Set CGRAM address command + address
	LCD_RS = 1;				// Back to data mode
	wait_lcd_rdy();
	clk_lcd(dta);
	clear_lcd();			// Back to normal mode (clear LCD)	
	}

void set_ch_bits(char dta, char cnt)		// Set character generator bits. Data, number of sets
	{
	char x;
	for(x = 0; x < cnt; x++)
		{
		wait_lcd_rdy();
		clk_lcd(dta);
		}
	}	

void set_chgen(void)		// Write bar symbols to HD44780 RAM character generator
	{
	wait_lcd_rdy( );
	LCD_RS = 0;				// Switch command mode
	clk_lcd(0 | ADRSET);	// Set CGRAM address command + address
	LCD_RS = 1;				// Back to data mode

	set_ch_bits(0, 1);
	set_ch_bits(0x10, 5);	// CGRAM 00 Blocks L > R
	set_ch_bits(0, 2);

	set_ch_bits(0, 1);
	set_ch_bits(0x14, 5);	// CGRAM 01
	set_ch_bits(0, 2);

	set_ch_bits(0, 1);
	set_ch_bits(0x15, 5);	// CGRAM 02 "Full block"
	set_ch_bits(0, 2);

	set_ch_bits(0, 1);
	set_ch_bits(0x01, 5);	// CGRAM 03 Blocks R > L
	set_ch_bits(0, 2);

//	set_ch_bits(0, 1);
//	set_ch_bits(0x05, 5);	// CGRAM 04
//	set_ch_bits(0, 2);

	set_ch_bits(0x1C, 1);	// CGRAM 04, PA-100D Temperature meter degree sign 
	set_ch_bits(0x14, 1);
	set_ch_bits(0x1C, 1);
	set_ch_bits(0, 5);
	
	set_ch_bits(0, 2);		// CGRAM 05 S-meter S9 marker
	set_ch_bits(0x10, 1);
	set_ch_bits(0, 1);
	set_ch_bits(0x10, 1);
	set_ch_bits(0, 3);

	set_ch_bits(0, 3);		// CGRAM 06 S-meter scale dot
	set_ch_bits(0x10, 1);
	set_ch_bits(0, 4);

	set_ch_bits(0, 2);		// CGRAM 07 S-meter scale bar
	set_ch_bits(0x10, 3);
	set_ch_bits(0, 3);

	clear_lcd();			// Back to normal mode (clear LCD)
	}

// Meter constants

#define DISP_LEN 8			// Display length (char)
#define FONT_W 6			// Font width
#define MAX_GRAPH (DISP_LEN * FONT_W)			// Graph length in pixels (5*16 = 80)

//const char scale_dots[16] = {6,32,6,32,6,32,6,32,7,32,7,32,7,32,7,32}; 	// For 16 char long graph
//const char scale_dots[8] = {6,32,6,32,7,32,7,32}; 						// For 8 long graph, standard
//const char scale_dots[8] = {6,6,6,6,7,7,7,7}; 							// 4 dots, 4 dash
const char scale_dots[8] = {6,6,6,6,5,7,7,7}; 								// 4 dots, S9 marker, 3 dash

// Draw 16 long S-meter, scale = 0...96 (48 visible steps)
// Draw 8 long S-meter, scale = 0...48 (24 visible steps)
// Buffer is used to avoid LCD flicker

void draw_s_meter(int len)
	{
	int x, whole, part, idx;
	char bar_buffer[DISP_LEN];				// Display buffer
	
	if(len > MAX_GRAPH)						// Cut to max
		len = MAX_GRAPH;

	whole = len / FONT_W;					// Calculate graph
	part = len % FONT_W;

	idx = 0;

	for(x = 0; x < DISP_LEN; x++)			// Copy scale dots to buffer
		{
		bar_buffer[idx] = scale_dots[x];
		idx++;
		}

	idx = 0;								// Draw graph to buffer	

	while(whole > 0)
		{
		bar_buffer[idx] = 0x02;				// Draw full blocks
		idx++;
		whole--;
		}

	if(part > 0)
		bar_buffer[idx] = Blockslr[part]; 	// Draw partial block to buffer

	for(x = 0; x < DISP_LEN; x++)			// Print meter buffer to LCD
		{
		lcdoutch(bar_buffer[x]);
		}
	}

// General I/O functions

// Display character
void lcd_putch(unsigned char c)
	{
	lcdoutch(c);
	}

// Display character in HEX format
void lcd_putchhex(unsigned char c)
{
	unsigned char temp;

	temp = c;
	c = (c >> 4);
//	c = (c < 10) ? c + 48 : c + 55;		// A shorter way of writing this - 5B4AIY
	c += (c < 10) ? 48 : 55;			// Although more concise, it results in extra code - 5B4AIY
	lcdoutch(c);
	temp = (temp & 0x0F);
//	temp = (temp < 10) ? temp + 48 : temp + 55;
	temp += (temp < 10) ? 48 : 55;		// Although more concise, it results in extra code - 5B4AIY
	lcdoutch(temp);						// A shorter way of writing this - 5B4AIY
}

// Display null terminated string
void lcd_putst(register const char *str)
	{
	while(*str) lcdoutch(*str++);		// A shorter way of writing this. - 5B4AIY
	}

// Display PA-100D meter formats

void disp_meter(double value, char mode)
	{
	switch(mode)
		{
		case 0:							// SWR Display
			sprintf(lcdpbuff, "SWR%4.2f", value / 100.0);
		break;

		case 'W':						// Output Power Display
			sprintf(lcdpbuff, "%6.1fW", value);
		break;

		case 'V':						// Battery Voltage Display
			sprintf(lcdpbuff, "%6.2fV", value / 1000.0);
		break;

		case 'A':						// PA Drain Current Display
			sprintf(lcdpbuff, "I%5.1f", value / 1000.0);
		break;

		case 'F':						// Fahrenheit Temperature Display
			sprintf(lcdpbuff, "T%4.0f%cF", value, 0xDF);	// Character 0xDF is degree symbol
		break;

		case 'C':						// Centigrade Temperature Display
			sprintf(lcdpbuff, "T%4.0f%cC", value, 0xDF);
		break;
		}
	lcd_putst(lcdpbuff);
	}

