// JUMA-PA100 service & setup routines
// Juha Niinikoski, OH2NLT, 08.07.2008

// Display clean up 11.11.2008
// Factory setup function corrected 16.12.2008

#include "juma-pa100.h"
#include "lcd-trx2.h"				// LCD specific definitions
#include <stdio.h>					// Added - 5B4AIY

// External references
// External functions

extern void beep(int, int);						// Generate beep, period, length
extern void ms_delay( unsigned int);			// Delay
extern int encoder_get(void);					// Read encoder
extern void clear_lcd(void);					// Clear LCD
extern void set_cur_lcd(unsigned char);			// Set cursor
extern void lcd_putst(register const char *);	// LCD put string
extern void disp_meter(double, char);			// Display meters - new version
//extern void disp_meter(int, int);				// Display meters
extern void power_off(void);					// Power down routine
//extern void display_s_meter(void);			// Display scaled S-meter
extern void disp_fwd_pwr(void);					// Display wattmeter
extern void disp_id(void);						// Display RF Amp Drain current
//extern void send_packet_to_74hc595(void);		// Main & PA board I/O
extern void check_alarms(void);					// Check alarms
extern void set_relays(void);					// Set gain & filter relays
extern void auto_band_select(void);				// Auto band select
extern void analog_measurements(void);			// Do analog measurements
extern void fan_control(void);					// Cooling fan control

extern int convert_adc12(unsigned int);			// Added - 5B4AIY
extern void set_factory_defaults(void);			// Added - 5B4AIY
extern void save_calval(void);					// Added - 5B4AIY
extern void save_defaults(void);				// Added - 5B4AIY

// External data

extern int batt;				// Power supply voltage scaled value, measured only when displayed
extern int fwd_adval;			// Forward & Reverse power raw values, measured only when TX
extern int rev_adval;
extern long fwd_pwr;			// Relative forward power
extern long rev_pwr;			// Relative reverse power
extern long out_pwr;			// Scaled output power = fwd_pwr - rev_pwr * scale
extern int id_cur;				// PA Drain current raw value
extern int service_mode;		// Service mode selected flag
extern int key;					// Copy of TX request input
extern unsigned int alarms;		// Alarm bits

extern unsigned int long_push;	// Long push timer for buttons
extern char lcdpbuff[];			// General purpose buffer for displays, etc

// EEPROM structures

#include "pa100_eeprom.h"		// Get EEPROM structure definitions

extern union
	{
	struct defval defval;
	unsigned int storage[sizeof(struct defval) / 2];
	}eeprom;

// System calibration values

extern union
	{
	struct calval calval;
	unsigned int ee[sizeof(struct calval) / 2];
	}cal;

#define MAX_SERVICE_PAGES 5		// Number of display pages

// Service & calibration functions, when tested move to own module

void service(void)
	{
	int cal_page = 0;
	unsigned long ultemp;
//	unsigned char lcdpbuff[20];	// LCD print buffer

	clear_lcd();
	eeprom.defval.pa_state = 1;	// Ensure we are in the operate mode

// Conditional start up text printout, print only if RS-232 = Test
 	if(eeprom.defval.serial_mode == 1)
		{
		printf("\n\rService & Calibration Mode");
		printf("\n\rDISPLAY/CONFIG = Parameter select, OPER = Save & Exit");
		}

	for(;;)
		{

// TX request
		if(KEY)					// Set TX request status
			key = 1;			// Copy I/O bit to status flag	
		else
			key = 0;

// Do measurements & check alarms
		analog_measurements();
		check_alarms();

// Auto band select
		auto_band_select();

// Set relays
		set_relays();

// Evaluate TX possibility, needed for power & IC adjustments

			if(key == 1)								// TX request
				{
				if(eeprom.defval.pa_state == 1)			// OPER
					{	
					if(alarms == 0 && eeprom.defval.band != NOT_KNOWN)			// No alarms, band selected
						TX_ON = 1;						// RF on
					}
				}
			else
				{
				TX_ON = 0;								// RF off
				}

// Cooling fan
		fan_control();

// Do A/D conversions for power meter
		fwd_adval = convert_adc12(FWD_PWR);
		rev_adval = convert_adc12(REV_PWR);

// Calculate Power
		fwd_pwr = (long)fwd_adval * (long)fwd_adval;	// Calculate relative power levels ( U^2)
		rev_pwr = (long)rev_adval * (long)rev_adval;
		out_pwr = fwd_pwr - rev_pwr;					// Output power = forward - reverse.

		if(DISP == 0)									// Select calibration function
			{
			cal_page++;

			if(cal_page > MAX_SERVICE_PAGES)			// Number of active pages
				cal_page = 0;

			beep(HZ392_01, 50);

			while(DISP == 0);

			ms_delay(BUTTON_DEBOUNCE);
			encoder_get();								// Flush encoder
			}

		switch(cal_page)								// Select major page to display
			{

// Battery voltage
			case 0: default:
				ultemp = (unsigned long)convert_adc12(BATT_CH) * cal.calval.batt_mult;	// Scale Battery voltage
//			batt = ultemp >> 8;														// Divide by 256, save scaled value
				batt = ultemp / 1000;
				set_cur_lcd(LINE1);
				lcd_putst("Supply:  ");
				disp_meter((double)batt, 'V');			// Display with two decimals
//			lcd_putst("V  ");

				cal.calval.batt_mult += encoder_get();	// Use coarse encoder to adjust

				if(cal.calval.batt_mult < 5000)
					cal.calval.batt_mult = 5000;

				if(cal.calval.batt_mult > 5500)
					cal.calval.batt_mult = 5500;

				set_cur_lcd(LINE2);
				sprintf(lcdpbuff, "Cal mult:%7li", cal.calval.batt_mult);	// Show scale multiplier
				lcd_putst(lcdpbuff);
			break;

// ID scaling
			case 1:
				set_cur_lcd(LINE1);
				lcd_putst("RF Amp.  ");
				disp_id();													// Display drain current

				cal.calval.id_mult += encoder_get();						// Use encoder to adjust

				if(cal.calval.id_mult < 750)
					cal.calval.id_mult = 750;

				if(cal.calval.id_mult > 2000)
					cal.calval.id_mult = 2000;

				set_cur_lcd(LINE2);
				sprintf(lcdpbuff, "Cal mult:%7li", cal.calval.id_mult);
				lcd_putst(lcdpbuff);
			break;

// Forward power scaling
			case 2:
				set_cur_lcd(LINE1);
				lcd_putst("Forward  ");
				disp_fwd_pwr();												// Display Watt meter

				cal.calval.fwd_pwr_mult += encoder_get();					// Use encoder to adjust

				if(cal.calval.fwd_pwr_mult < 750)
					cal.calval.fwd_pwr_mult = 750;

				if(cal.calval.fwd_pwr_mult > 1500)
					cal.calval.fwd_pwr_mult = 1500;

				cal.calval.rev_pwr_mult = cal.calval.fwd_pwr_mult;			// Use same value for reverse power

				set_cur_lcd(LINE2);
				sprintf(lcdpbuff, "Cal mult:%7li", cal.calval.fwd_pwr_mult);
				lcd_putst(lcdpbuff);
			break;

// Beep length
			case 3:
				set_cur_lcd(LINE1);
				sprintf(lcdpbuff, "Beep len 0 = OFF");
				lcd_putst(lcdpbuff);

				set_cur_lcd(LINE2);
				cal.calval.beep_len += encoder_get();						// Use coarse encoder to adjust

				if(cal.calval.beep_len < 0)
					cal.calval.beep_len = 0;

				if(cal.calval.beep_len > 100)
					cal.calval.beep_len = 100;

				sprintf(lcdpbuff, "Beep:%9imS", cal.calval.beep_len);		// Show current setup
				lcd_putst(lcdpbuff);
			break;

// TRX2 Poll rate
			case 4:
				set_cur_lcd(LINE1);
				lcd_putst("TRX2 Poll Rate  ");

				cal.calval.q_timer += (encoder_get() * 100);

				if(cal.calval.q_timer > MAX_POLL_TIMER)
					cal.calval.q_timer = MAX_POLL_TIMER;

				if(cal.calval.q_timer < 0)
					cal.calval.q_timer = 0;

				set_cur_lcd(LINE2);
				lcd_putst("Rate:");

				if(cal.calval.q_timer)
					{
					sprintf(lcdpbuff, "%7.1f Sec", ((double)cal.calval.q_timer) / 1000);	// Improved display - 5B4AIY
					lcd_putst(lcdpbuff);
					}
				else
					lcd_putst("        OFF");
			break;

// Factory defaults
			case 5:
				set_cur_lcd(LINE1);
				lcd_putst("Push OPER long =");
				set_cur_lcd(LINE2);
				lcd_putst("factory defaults");
			break;

			}				// End switch

// Read switches
			if(cal_page == 5)											// Restore defaults only when cal_page == 5
				{
				if(OPER == 0)											// Test SW2 (SET)
					{
					long_push = LONG_PUSH;								// Set long push timer

					while(OPER == 0)									// Detect long push
						{
						if(long_push == 0)								// Detect long push, restore factory setup
							{
							set_factory_defaults();						// EEPROM empty, set factory defaults
							save_calval();								// Save calibration values
							save_defaults();							// Save user defaults

							beep(HZ466_85, cal.calval.beep_len*10);		// Long beep for save
							service_mode = 0;							// Back to normal operation
							clear_lcd();
							lcd_putst("Factory setup OK");
		
 							if(eeprom.defval.serial_mode == 1)			// Conditional print only if RS-232 = Test
								printf("\n\rFactory defaults restored");

							ms_delay(1500);								// Show message

							while(OPER == 0);
							return;										// Exit service mode
							}
						}		// End while
					}			// End if(OPER == 0)
				}				// End (cal_page == 6)
			else
				{
				if(OPER == 0)											// Test SW2 (SET) when  cal_page != 6
					{
					save_calval();										// Save changes
					beep(HZ466_85, cal.calval.beep_len);				// Short beep for return to normal operation
					service_mode = 0;									// Back to normal operation
					clear_lcd();
					lcd_putst("Calibr. Saved   ");

					if(eeprom.defval.serial_mode == 1)					// Conditional print only if RS-232 = Test
						printf("\n\rCalibration settings saved to EEPROM");

					while(OPER == 0);
					return;												// Exit service mode
					}
				}			// End else

		if(PWR_SW)
			power_off();			// Go shut down, do not save changes

		}		// End for(;;)
	}			// End function

