// dsPIC30 timers PWM DAC for JUMA-PA100
// Juha Niinikoski, OH2NLT 06.07.2008

// Microchip C30 compiler conversion

// JUMA-PA100 timer & pwm system

// Timer & PWM unit usage

// TMR1 is used for frequency counter
// TMR2 is used for tone generation, IRQ driven system, RD1/OC2 = tone out
// TMR3 is used for 1ms timer tick IRQ and for LCD PWM dac timebase
// OC3 output is for LCD back light control
// OC4 output is for LCD contrast
// OC3 and OC4 are connected to TMR3

// TMR4 
// TMR5 used for delays (separate module)

// Encoder simulator repeat logic added 13.08.2008

//#define TICK_PERIOD 7500			// 1ms tick
#define TICK_PERIOD (FCY/1000)		// 1ms tick

#include "juma-pa100.h"

unsigned int freq;					// Frequency counter
unsigned int max_freq;
unsigned int max_freq_delay;
unsigned int max_freq_set;
#define MAX_FREQ_DELAY 50;			// Peak hold delay
unsigned int band_query_timer;		// Band data query timer 
 
static int blink_counter; 			// Blink cycle (ms)
static unsigned int ms_counter;		// 1 second divider, 1000ms counter
static unsigned int tone_counter; 	// Beep tone length (ms)
static unsigned int busy_counter;	// Tone generator busy(ms), busy if != 0
static int last_cycle;				// For click less end trick

unsigned int t_stamp;				// 1ms free running time stamp
int cmd_timeout;					// RS-232 command timeout, 0 = timeout, negative values = state flags.

int blink;							// Alarm blink flag
unsigned int long_push;				// Long push timer, count from set value to zero, global visibility

int decay_counter;					// Power meter slow decay

// Encoder simulator
static unsigned int up;				// UP button shift register
static unsigned int dn;				// DOWN button shift register

static int up_pushed;				// Up pushed status
static int dn_pushed;				// Down pushed status
static int rep_up;					// Repeat timers (ms)
static int rep_dn;
static int rep_up_dly;				// Repeat start delay (ms)
static int rep_dn_dly;

int enc;							// Encoder accumulator

// External variables
extern int seconds;					// Free running second counter

// EEPROM structures
#include "pa100_eeprom.h"			// Get EEPROM structure definitions

extern union
	{
	struct defval defval;
	unsigned int storage[sizeof(struct defval) / 2];
	}eeprom;

// Simulate encoder with UP / DOWN buttons
// Read encoder
int encoder_get(void)
	{
	int temp;

	temp = enc;
	enc = 0;
	return(temp);
	}

// Band selector
extern const unsigned int band_limits[10];	// Band limits defined in the main
int band_bins[10];							// Found samples
int band_meas_ready;						// Band select measurement ready
int freq_sample_ctr;						// Sample counter

// Initialise timer & PWM system
void init_timers_pwm(void)
	{
	freq_sample_ctr = F_SAMPLES;

	ms_counter = 1000;
	OC3CON = 0;								// First turn off the module if it was on
	OC4CON = 0;

// LCD PWM DAC setup
	OC3RS = 100;							// Some small value for back light
	OC3R = 100;
	OC3CON = 0x000E;						// Use TMR3, OC3 = PWM mode

	OC4RS = 2000;							// Something for contrast to make display visible
	OC4R = 2000;
	OC4CON = 0x000E;						// Use TMR3, OC4 = PWM mode

	PR3 = TICK_PERIOD - 1;					// Set PWM cycle, TMR3 period register, 1000Hz/1ms
	T3CONbits.TON = 1;						// Start TMR3
	_T3IE = 1;								// Enable TMR3 tick IRQ

// Timer 1 for frequency counting
	_T1IE = 0;
	T1CON = 0x0002;							// External clock, no sync
//	T1CON = 0x0000;							// External clock, no sync

	max_freq_set = MAX_FREQ_DELAY;
	}

// PWM set functions

void set_pwm3_dac(unsigned int pwm)			// LCD back light
	{
	OC3RS = pwm;
	}

void set_pwm4_dac(unsigned int pwm) 		// LCD contrast
	{
	OC4RS = pwm;
	}

// Tone generator, TMR2
void __attribute__((interrupt)) _T2Interrupt(void)
	{
	switch(last_cycle)						// Tone generator state machine
		{
		case 0: default:					// Tone output off, tri-state
			TONE_TRIS = 1;					// Tri-state output
			_T2IE = 0;						// Disable TMR1 IRQs, Tone generator off
		break;

		case 1:								// Last half cycle
			TONE_OUT = !TONE_OUT;
			PR2 = PR2 >> 1;					// Next = half cycle
			last_cycle = 0;					// Flag for shut down
		break;

		case 2:								// Last full cycle, start shut down
			TONE_OUT = !TONE_OUT;			// Inverse tone out on every cycle

			if(TONE_OUT != 0)				// End with down going half cycle
				last_cycle = 1;				// Next = last full cycle
		break;

		case 3:								// Generate sound
			TONE_OUT = !TONE_OUT;			// Inverse tone out on every cycle
		break;
		}
	_T2IF = 0;
	}

// Tone set functions
void tone_on(int tone)						// Set tone, set value = 7,5MHz / tone(Hz) * 2
	{
	last_cycle = 3;							// Not last cycle, run state
	PR2 = tone - 1;							// Set tone period
	TMR2 = PR2 >> 1;						// Start with half period, reduce click
	TONE_OUT = 0;							// Start allways same phase
	TONE_TRIS = 0;							// Enable output
	T2CONbits.TON = 1;						// Start timer
	_T2IF = 0;								// Clear old pending interrupt, start with clean half cycle
	_T2IE = 1;								// Enable TMR1 IRQs, Tone generator on
	}

void tone_off(void)							// Stop tone generator
	{
	last_cycle = 2;							// Set flag for stop tone, last half cycle
	}

void beep(int tone, int duration)
	{
	if(busy_counter == 0 && duration > 0)	// Start tone only if tone generator free & duration specified
		{
		tone_counter = duration;			// Set tone length
		tone_on(tone);						// Set tone pitch & start play
		}
	}

// Frequency counter test

unsigned int get_fcounter(void)
	{
	return(freq);
	}

// IRQ code
// 1ms Tick timer
void __attribute__((interrupt)) _T3Interrupt(void)
	{
	int x;

	_T3IF = 0;

//	IRQ_TEST = 1;							// Timing test

// Frequency counter
	T1CONbits.TON = 0;						// Stop counting
	freq = TMR1;							// Save frequency
	TMR1 = 0;								// Reset counter
	T1CONbits.TON = 1;						// Start counter

// Band select		
	if(band_meas_ready == 0)				// Run only when previous measurement is used
		{
		for(x = 0; x < 10; x++)					// Search bin
			{
			if(freq <= band_limits[x])
				break;
			}
		band_bins[x]++;						// Sample found

		freq_sample_ctr--;					// Count samples
		
		if(freq_sample_ctr == 0)
			band_meas_ready = 1;			// Set measurement ready flag
		}

// Tone duration counter
	if(tone_counter != 0)
		{
		tone_counter--;

		if(tone_counter == 0)
			{
			tone_off();						// Run stop with half cycle logic
			}
		}

// Push button timer
	if(long_push != 0)						// Decrement long push timer if active
		long_push--;

// General timing
	ms_counter--;							// Run 1s free-running counter

	if(ms_counter == 0)
		{
		ms_counter = 1000;
		seconds++;
		}

// Blink flag
	blink_counter--;

	if(blink_counter < 0)
		{
		blink_counter = BLINK_RATE;

		if(blink == 0)
			blink = 1;
		else
			blink = 0;
		}

// Power meter decay counter
	if(decay_counter != 0)
		decay_counter--;

// Band data query timer
	if(band_query_timer != 0)
		band_query_timer--;

// In the PA100 we do not have rotary encoder.
// Encoder functions are simulated, with UP & DOWN buttons

// Do debounce shift registers
	up = up << 1;					// UP button, encoder ++

	if(UP == 0)
		up = up | 0x0001;
	else
		up = up & 0xFFFE;

	dn = dn << 1;					// DOWN button, encoder --

	if(DN == 0)
		dn = dn | 0x0001;
	else
		dn = dn & 0xFFFE;

// Test pushed condition

// UP
	if(up_pushed == 0)				// Not pushed
		{
		if(up == 0xFFFF)			// Test if de-bounce shift register all ones
			{
			up_pushed = 1;			// Set pushed
			rep_up_dly = REP_DLY;	// Set repeat start timer
			enc++;					// Increment encoder
			}
		}
	else							// Pushed
		{
		if(up == 0x0000)			// Test if de-bounce shift register all zeroes
			{
			up_pushed = 0;			// Clear pushed
			rep_up_dly = 0;			// Clear repeat
			rep_up = 0;
			}
		}

// DOWN
	if(dn_pushed == 0)				// Not pushed
		{
		if(dn == 0xFFFF)			// Test if de-bounce shift register all ones
			{
			dn_pushed = 1;			// Set pushed
			rep_dn_dly = REP_DLY;	// Set repeat start timer
			enc--;					// Increment encoder
			}
		}
	else							// Pushed
		{
		if(dn == 0x0000)			// Test if de-bounce shift register all zeroes
			{
			dn_pushed = 0;			// Clear pushed
			rep_dn_dly = 0;			// Clear repeat
			rep_dn = 0;
			}
		}

// Repeat start timers
	if(rep_up_dly != 0)				// Check if UP button start repeat timer is running
		{
		rep_up_dly--;

		if(rep_up_dly == 0)			// Start repeat action
			rep_up = REP;
		}

	if(rep_dn_dly != 0)				// Check if DOWN button start repeat timer is running
		{
		rep_dn_dly--;

		if(rep_dn_dly == 0)			// Start repeat action
			rep_dn = REP;
		}

// Repeat generator timers
	if(rep_up != 0)					// UP button repeat counter going ?
		{
		rep_up--;

		if(rep_up == 0)
			{
			rep_up = REP;			// Reload timer
			enc++;					// Add encoder accumulator
			}
		}

	if(rep_dn != 0)					// DOWN button repeat counter going ?
		{
		rep_dn--;

		if(rep_dn == 0)
			{
			rep_dn = REP;			// Reload timer
			enc--;					// Subtract encoder accumulator
			}
		}

	t_stamp++;						// Increment time stamp
//	IRQ_TEST = 0;					// Timing test
	}

